/*Fifo.h
Kernel Fifo functions
*/
#ifndef FIFO_H
#define FIFO_H

#include <GNU_defs.h>
#include <stdlib.h>

/*! @defgroup Fifo Fifo Module

  This module creates simple fifos for storing data in a locking circular queue. It is used internally by many of the other
  modules. These fifo's use locks to prevent concurrent access. If concurrent access occurs(via and interrupt or scheduled event)
  the fifo will fail to store/retrieve the data in question and will return 0 to indicate as such.
 */

/*@{*/

/*!The fifo structure

*/
struct KFifo{
volatile u8 *firstin;/*!pointer to data to be pulled*/
volatile u8 *lastin;/*!pointer to data to store*/
void *top;/*!pointer to highest fifo address available*/
void *bottom;/*!pointer to lowest address available*/
volatile size_t mem_avail; /*!unused memory in fifo*/
size_t size;
int lock;
//int executions;
};

typedef struct KFifo KFifo;

#define FIFOEMPTY(fifo) (fifo->mem_avail==fifo->size)
#define FIFOPRNTSTATS(fifo) {GENDEBUG("top = %d\r\n",(long)fifo->top);GENDEBUG("bottom = %d\r\n",(long)fifo->bottom); GENDEBUG("mem_avail = %d\r\n",(long)fifo->mem_avail);GENDEBUG("size = %d\r\n",(long)fifo->size);}


/*! Create Initialize a new Fifo structure around an array
InitFifo takes an existing array and wraps a fifo structure around it.
Once this has been done the other fifo functions can be used to push and pull data out of it.
@param Fifo pointer to the fifo structure to initialize
@param size  the size of the fifo (for most cases this should be sizeof(array)
@param array the array the fifo will be using for storing/retrieving data
*/
int __attribute__((far))InitFifo(KFifo *Fifo, size_t size, u8 *array);


/*!Push a byte into a fifo
@param Fifo pointer to the fifo to use
@param data the byte to push
@return SUCCESS
*/
int __attribute__((far))FifoPush(KFifo *Fifo, const char data);

/*!Push a block of data into a fifo
@param Fifo pointer to the fifo to push data into
@param buffer pointer to the data to push into the fifo
@param count the number of bytes to push into the fifo from buffer
@return the number of bytes stored in the fifo
*/
ssize_t __attribute__((far))FifoPushBlock(KFifo *Fifo, const char *buffer, size_t count);

/*!Pull a byte from a fifo
This function doesn't give any indication of success or failure, it simply returns 0 if no data is available
(of course zero could be a valid data byte in the fifo)
If you need to know if it succeeded or not use FifoPullBlock and just pull 1 byte
@param Fifo pointer to the fifo to pull from
@return the byte pulled from the fifo
@see FifoPullBlock
*/
char __attribute__((far))FifoPull(KFifo *Fifo);

/*!Pull a block of data from a fifo
@param Fifo pointer to the fifo to pull from
@buffer pointer to an array to store the data in
@count number of bytes to pull
@return number of bytes pulled
*/
ssize_t __attribute__((far))FifoPullBlock(KFifo *Fifo, char *buffer, size_t count);

/*!Destroy a fifo
Since this fifo implementation used static memory for everything this function is just an empty shell
If you want to clear a fifo just re-initialize it.
@param Fifo pointer to the fifo to destroy
@return SUCCESS
*/
int __attribute__((far))DestroyFifo(KFifo *Fifo);

/*@}*/

#endif

