#ifndef IOCCODE
#define IOCCODE
#include <GNU_defs.h>
#include <realtime.h>

/*! @defgroup IOC   Enhanced Capture Timer Module
This module provides routines to control the Enhanced Capture Timers or ICTs of the IPAC.
This corresponds to PORTT of the Motorola DG128 and is the same port as  PORT1 when references as a GPIO digital port.
These 8 lines can be individually configured as PWMs or 16 bit down counters using the functions in this module.
*/

/*@{*/

/*!IC edges*/
#define NOE   0  /*!no input capture*/
#define RISEE 1 /*!rising edge*/
#define FALLE 2 /*!falling edge*/
#define ANYE 3 /*!rising and falling edge cause the counter to decrement*/


/*!IC configuration commands (passed via the flags variable)*/
/*!automatic reload of threshold value*/
#define ARELOAD 2
/*!automatic stop when counter hits zero*/
#define ASTOP   4

/*!callback type for the counter callback
These counter callbacks can be registered to happen when a counter hits zero.
@param counter ICT number that triggered the callback
@param privatedata generic pointer to a users data
@see IC_CallbackZ
*/
typedef void __attribute__((far))(*count_callback)(int counter, void *privatedata);


typedef struct count_notify{
count_callback callback;
void *privatedata;
}count_notify;


typedef struct Count_Struct
{
u16 counter[8]; /* 8 16 bit counters */
u16 countstart[8]; /* 8 16 bit counters */
u8  countctl[8];
u16 countreload[8];//reload values
u8 rcount; //bitflag whether or not to reload
u8 scount; //bitflag whether or not to stop the counter
count_notify notify[8];
}Count_Struct;


/*! PWM emulation
This function turns the specified ICT into a PWM
Typically it is used to emulate PWMs between 1 and 10KHz at varying duty cycles.
This is a software PWM so maximum frequencies and duty cycles vary with system load, however low KHz frequencies
should support all duty cycles in almost any system load
This function does not start the ICT, just configures it. IOCstart must be used for that.

@param Duty the duty cycle 0-100
@param ioc the ICT to use for a PWM
@frequency the frequency to run at.
@returns SUCCESS or FAILURE, FAILURES can occur if the frequency/duty cycle combination cannot be achieved with 16 bit
counter reloads. This will typically happen at frequencies lower than 300HZ
@see IOCstart
*/
int __attribute__((far))IOCPWMem(int Duty, int ioc, unsigned long frequency);

/*! starts the IOC (PWM'd ICT) that has been pre-configured by IOCPWMem
@param ioc
@see IOCPWMem
*/
void __attribute__((far))IOCstart(taskdata ioc);


/*!stops an IOC thats currently running
@param ioc
*/
void __attribute__((far))IOCstop(taskdata ioc);


/*!turn an ICT into a counter
This function is technically misnamed, ICT_IC would make more sense.
IOC_IC configures an ICT as a 16 bit down counter with several different options
This function has a higher level, the init modules function, Counter_Ipac, which stores
all the configuration data sent to it in global variables.
This function only configures a counter it does not turn it on, for that use startcounter

@param ict the ict to configure
@param trigger what type of edge causes the counter to decrement see IC edges in the defines
@param reload the reload value and starting counter value
@param flags control flags, a bitwise combination which can contain ARELOAD or ASTOP
@see startcounter
*/
int __attribute__((far))IOC_IC(int ict, u8 trigger, u16 reload, u8 flags);

/*!Install a zero cross callback on an IC(ICT configured as a counter)
@param ic the IC to install the callback on.
@param callback the callback function to install
@param privatedata a pointer to user data
@return SUCCESS
*/
int __attribute__((far))IC_CallbackZ(int ic, count_callback callback, void *privatedata);


/*!Remove an IC callback
@param ic the IC callback to remove
@return SUCCESS
*/
int __attribute__((far))IC_Callback_off(int ic);

/*!Return the value last loaded into a counter
This is either the starting value or the value reloaded after a zero crossing(which could be 0xffff or the reload value depending
on the flags) This can be useful for computing how many counts have elapsed in situations where the reload value may vary.
@param counter the IC to reference
@return the value of the counter that was last loaded to it.
*/
u16 __attribute__((far))CountStart(int counter);

/*!Read the current value of a counter
This reads the current value of the counter
@param counter the IC to read
@return the value of the counter
*/
u16 __attribute__((far))ReadCounter(int counter);

/*!Turn on an IC
@param ic the IC to start
@return SUCCESS
*/
int __attribute__((far))startcounter(int ic);
/*!Turn off an IC
Counters that are turned off will not lose their configuration and can be turned back on at any time with startcounter
@param ic the IC to stop
@return SUCCESS
*/
int __attribute__((far))stopcounter(int ic);

/*@}*/
#endif
