/*
 * key.h
 * Header file for Keypad drivers for the EMAC uP-HC12
 */

/*! @defgroup KeyPad Keypad functions

This module provides functions for receiving data from the Keypad of the IPAC.
The Keypad module uses the realtime scheduler to scan the keypad for data.
When data is received it's placed in a fifo and if a callback is registered it's called.
Data can be read from the fifo at any time using the Keyrx function.
*/

#ifndef _KEY_H_
#define _KEY_H_

#include <GNU_defs.h>

#define KEYCODE


/*@{*/
#define KEYCOL 6
#define KEYROW 4

#define GETROW(key) (key>>4)
#define GETCOL(key) (key&0x0f)

/*!a keypad callback type of function can be registered to be called whenever a key is pressed.
*/
typedef void __attribute__((far))(*key_callback)(void *);


/*! Receive data from the Keypad fifo
Pull data from the keypad fifo. To keep this module generic and not rely on a specific keypad, no keypad matix is used.
instead the character returned indicates the row and column that the character belongs to.
The row is the top nibble column is the bottom nibble; The macros GETROW and GETCOL can be used for this purpose
@param count the number of characters to pull from the fifo
@param pointer to an array to store the characters in.
@return the number of characters placed into buffer
@see GETROW
@see GETCOL
*/
ssize_t __attribute__((far))Keyrx(int count, u8 *buffer);

/*! block until a key is pressed
This function polls the keypad fifo until data appears. That character is then returned.
@return the character received
*/
char __attribute__((far))Key_getch(void);

/*!install a keypad callback
KeyCall can be used to install a callback function on the keypad module.
Whenever a key is pressed this function is called with the arguement privatedata.
The callback doesn't return the key, this must be read with Keyrx
A Keypad Callback can be removed by calling KeyCall again with NULL as the callback arg
Note: this function briefly suspends the real-time interrupt when run.
@param callback a key_callback function that will be called whenever a key is pressed
@param privatedata a pointer to anything, this pointer will be passed to the callback when it is run
*/
int __attribute__((far))KeyCall(key_callback callback, void *privatedata);

/*@}*/

#endif
