/***************************************************************************
                          can.h  -
              CAN control functions .
                             -------------------
    begin                : Tue Oct 14 2003
    email                : support@emacinc.com
 ***************************************************************************/


/*! @defgroup CAN CAN Module
This module provides functions for controlling the CAN (Control Area Network)
Of an E12 device. 
*/
/*@{*/

#ifndef CAN_H
#define CAN_H

#include <gnu_defs.h>
#include <string.h>


/*This is the function around which all CAN register references are 
 * wrapped. It takes the base CAN address from the variable "base" 
 * in the current context.
 */
static inline u8 *caddr(int x, volatile u8 *base){
	return ((u8 *)(x + (unsigned long)base));
}

#define MODULE_OFFSET(x) *(caddr(x,base))
#define MARRAY_OFFSET(x)  (caddr(x,base))

#define CANCTL0     MODULE_OFFSET(0x0)
	#define	INITRQ	0x01
	#define CSWAI	0x20

#define CANCTL1     MODULE_OFFSET(0x1)
	#define CANE	0x80
	#define CLKSRC	0x40
	#define LISTEN	0x10
	#define LOOPB	0x20
	#define INITAK	0x01

#define CANBTR0     MODULE_OFFSET(0x2)
#define CANBTR1     MODULE_OFFSET(0x3)
#define CANRFLG     MODULE_OFFSET(0x4)
	#define CSCIF	0x40
	#define RXF	0x01
	#define OVRIF	0x02	
	
#define CANRIER     MODULE_OFFSET(0x5)
	#define WUPIE	0x80
	#define RXFIE   0x01

#define CANTFLG     MODULE_OFFSET(0x6)
#define CANTIER     MODULE_OFFSET(0x7)
#define CANTARQ     MODULE_OFFSET(0x8)
#define CANTAAK     MODULE_OFFSET(0x9) 
#define CANTBSEL    MODULE_OFFSET(0xa)
#define CANIDAC     MODULE_OFFSET(0xb)
#define CANRXERR    MODULE_OFFSET(0xe)
#define CANTXERR    MODULE_OFFSET(0xf)
#define CANIDAR0    MODULE_OFFSET(0x10)
#define CANIDAR1    MODULE_OFFSET(0x11)
#define CANIDAR2    MODULE_OFFSET(0x12)
#define CANIDAR3    MODULE_OFFSET(0x13)
#define CANIDMR0    MODULE_OFFSET(0x14)
#define CANIDMR1    MODULE_OFFSET(0x15)
#define CANIDMR2    MODULE_OFFSET(0x16)
#define CANIDMR3    MODULE_OFFSET(0x17)
#define CANIDAR4    MODULE_OFFSET(0x18)
#define CANIDAR5    MODULE_OFFSET(0x19)
#define CANIDAR6    MODULE_OFFSET(0x1a)
#define CANIDAR7    MODULE_OFFSET(0x1b)
#define CANIDMR4    MODULE_OFFSET(0x1c)
#define CANIDMR5    MODULE_OFFSET(0x1d)
#define CANIDMR6    MODULE_OFFSET(0x1e)
#define CANIDMR7    MODULE_OFFSET(0x1f)

#define CANRXFG	    MARRAY_OFFSET(0x20)
#define CANTXFG     MARRAY_OFFSET(0x30)

//status macros
#define CAN_RUNNING    	 CANCTL1&INITAK
#define CAN_RXBUF_FULL	 CANRFLG&RXF 
#define CANOVERRUN	 CANRFLG&OVRIF

#define QUEUE_LENGTH	16		/*rx messages queue*/
#define CANTXQ_LENGTH	5		/*tx message queue*/

#define CAN_MSG_LENGTH	8

#define MSG_RTR		(1<<0)		/**< RTR Message */
#define RTR		0x10

/*!can message structure
*/
typedef struct canmsg{
    // flags, indicating or controlling special message properties 
    u8 flags;/*!CAN message flags, indicating or controlling special message properties */    
    u16 id;	 /*!ID number, currently standard only*/ 
    u8 length;/*!number of bytes in the CAN message*/ 
    u8 data[CAN_MSG_LENGTH]; /*! data, 0...8 bytes */
} canmsg;

/*!can queue structure
 * This structure must be initialized and hooked to a canmsg buffer before
 * initializing the CAN hardware. Users are responsible for allocating enough 
 * memory to hold the required number of CAN messages.
 * A number of queue functions are provided below.
 * The simplist method for using the queue is using standard push/pull functions
 * but this requires memory copying since the queues are represented by arrays and
 * not true dynamic linked lists. A more efficient but mroe complicated method is the 
 * use the data directly from the array, the remove it with CAN_RX_REMOVE.
 * A similar approach can be taken with pushing data by retrieving the tx pointer,
 * to write to with CAN_WRITE_BUFFER, copying data into it and calling CAN_ENQEUE
*/
typedef struct canqueue{
	int in;/*!index to next available in push slot*/  
	int out;/*!index to next available in pull slot*/ 
	int size;/*!total queue size*/ 
	int messages;/*!number of messages in the queue*/ 
	canmsg *queue;
}canqueue; 

/*!can module structure
 * This structure must be initialized by the user before
 * initializing the CAN hardware. See the CANflector program for a simple
 * example
*/
typedef struct can_module{	
	unsigned long long id;/*!CAN ID, full extended functionality not yet implemented*/  
	unsigned long long mask;/*!CAN mask, full extended functionality not yet implemented*/  
	unsigned long baud;/*!CAN bitrate in KHz, acceptable values are 250,500, and 1000*/  
	int index;/*!CAN module index, for the IPAC this must be 0, set by CAN_device call*/  
	volatile u8 *base;/*!base address, auto set by CAN_device call*/  
	canqueue tx,rx;/*!queues for tx and rx*/
}can_module;

/*!empty a modules queues, called during CAN initialization to zero out a queue.
 * also could be used to quickly tump a queues contents.
 * @param CAN pointer to the CAN module to empty
*/
static inline void CAN_EMPTYQUEUE(can_module *CAN){
	CAN->tx.messages = CAN->tx.in = CAN->tx.out = CAN->rx.messages =
	 CAN->rx.in = CAN->rx.out = 0;
}

/*!retrieve a current queue pointer for reading from a queeu
 * @param q pointer to the queue to read from
 */
static inline canmsg *CAN_READ_BUFFER(canqueue *q){
	return (&q->queue[q->out]);
}

/*!retrieve a current queue pointer for writing to a queue
 * @param q pointer to the queue to write to
 */
static inline canmsg *CAN_WRITE_BUFFER(canqueue *q){
	return (&q->queue[q->in]);
}

/*!enqueue a message into the queue. This moves the queue pointers forward, 
 * making the message at the current input index actual data in the queue. 
 *@param q pointer to the queue enqueue into 
 */
static inline void CAN_ENQEUE(canqueue *q){
	q->in++;
	if(q->in%q->size)q->in=0;
	q->messages++;
}

/*!remove the CAN message on the top of a queue(but don't return it)
 * this is called by the pull function after copying the message out
 *@param q pointer to the queue to remove from
 */
static inline void CAN_RX_REMOVE(canqueue *q){
	q->out++;
	if(q->out%q->size)q->out=0;
	q->messages--;
}

/*!push a message into a queue
 * @param q pointer to the queue to push into
 * @param msg pointer to the message to push
 */
static inline void CAN_PUSH(canqueue *q,canmsg *msg){
	memcpy(CAN_WRITE_BUFFER(q),msg,sizeof(canmsg));
	CAN_ENQEUE(q);
}
/*!pull a message from a queue and copy it into a message buffer
 * @param q pointer to the queue to pull from
 * @param msg pointer to a buffer for holding the message
 */
static inline void CAN_POP(canmsg *msg,canqueue *q){
	memcpy(msg,CAN_READ_BUFFER(q),sizeof(canmsg));
	CAN_RX_REMOVE(q);
}

/*!enable CAN trasmission
 * This will result in an immediate interrupt from the CAN module which will pull
 * all the messages in it's transmission queue and send them, when finished, it will
 * automatically disable transmission again.
 */
static inline void CAN_TX_ENABLE(can_module *CAN){
	volatile u8 *base = CAN->base;
	CANTIER=7;
}
/*!disable CAN trasmission, used for locking*/
static inline void CAN_TX_DISABLE(can_module *CAN){
	volatile u8 *base = CAN->base;
	CANTIER=0;
}
/*!enable CAN reception, this will allow CAN rx interrupts which will
 * fill the CAN structures rx queue*/
static inline void CAN_RX_ENABLE(can_module *CAN){
	volatile u8 *base = CAN->base;
	CANRIER|=RXFIE; 
}
/*!disable CAN reception, this will automatically happen if data is received but
 * not space is avaialable in the queue*/
static inline void CAN_RX_DISABLE(can_module *CAN){
	volatile u8 *base = CAN->base;
	CANRIER&=~RXFIE; 
}

/*!print out a CAN message to a serial port
 * @param comport serial port number to display the message on
 * @param msg pointer to the message to display
 */
static inline void can_print_message(int comport,canmsg *msg){
	ser_printf(comport,"canmsg %u\r\n",(long)msg->id);
	ser_printf(comport,"flags-> %u\r\n",(long)msg->flags);
	{int i;
		ser_printf(comport,"data: ");
		for (i=0;i<msg->length;i++) 
			ser_printf(comport,"%u ",(long)msg->data[i]);
	}ser_printf(comport,"\r\n");
}

/*!return a CAN device with a given index. 
 * @param index CAN module index, for the IPAC this must always be zero
 * @returns pointer to the can module at index, with base and index elements set up.
 */
can_module * __attribute__((far))CAN_device(int index);

/*!Start up a CAN module, this should be called after the structure and it's queues
 * are set up.
 * @param CAN The can module retrieve from CAN_device after it has been initialized
 * @returns 0
 */
int __attribute__((far))CANStart(can_module *CAN);

/*@}*/
#endif
