#ifndef INIT_H
#define INIT_H

#include <GNU_defs.h>
#include <math.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <stdarg.h>
#include <ser.h>
#include <Fifo.h>
#include <realtime.h>
#include <ioc.h>
#include <pwm.h>
#include <Relay.h>
#include <key.h>
#include <atod.h>
#include <lcd.h>
#include <ssi.h>
#include <DS1305RTC.h>
#include <LTC1290.h>
#include <tlv5614.h>
#include <clocks.h>
#include <eeprom.h>


/*init.h
board specific defines,initializations, and tests
*/

/*! @defgroup INIT Initialization routines and debug utilities

    The IPAC module provides some high level calls to the other drivers.
 */




/*@{*/

/*!
A data structure to store global configuration data about the PWMs
This  structure is used to create PWMdata[PWMSYS] which is updated by
PWM_Ipac whenever a PWM is created/modified.
PWMdata ports get reset to zero when they are initialized or when they are turned into counters
*/
typedef struct PWMdatastruct
{
int duty;
unsigned long frequency;
}PWMdatastruct;

extern PWMdatastruct PWMdata[PWMSYS];

/*! Used to store information about counter configurations, here just the trigger and flags.
The current count/reload information can be retreived with counter calls. This structure
is updated by Counter_Ipac whenever an IC is created/modified.
CounterData ports get reset to zero when they are initialized or when they are turned into PWMs
*/
typedef struct Countdatastruct
{
int trigger;
int flags;
}Countdatastruct;

extern Countdatastruct CountData[IOCNUMPORTS];

/*! This structure contains only one element, output.
It keeps track of the current DtoA output percentage.
It represents the duty cycle of a filtered PWM.
*/
typedef struct DtoAdatastruct
{
int output;
}DtoAdatastruct;



/*!Initializes the board
This sets up and intializes:
The status LED
The Pll
The 2 serial ports
The 2 internal AtoD modules
Disconnects IRQ from E1 and enables IRQ's (X is not enables)
The realtime scheduler
The IOC PWM data structures
The IC counter data structures
The DtoA data structures
Starts the global down counter.
Initializes the PLD
Initializes the Keypad
Initializes the PLL
Initializes the SPI and external RTC
Initializes the eeprom
*/
int __attribute__((far))init(void);

/*!Initialize the Ipacs PWM data structures
Called by IpacInit
@return SUCCESS
*/
int __attribute__((far))PWM_Ipac_Init(void);

/*!Configure and start a PWM
Currently this function is just used to set up IOC PWMs.
It configures an ICT to be a PWM, starts it, and stores the information about it in PWMdata[PWMSYS]
@param ict The ICT to use for a PWM
@param duty the PWM duty cycle
@param frequency the PWM frequency
@return SUCCESS or FAILURE
*/
int __attribute__((far))PWM_Ipac(int ict,int duty,unsigned long frequency);


/*!Configure and start a 16 bit down counter
Currently this function is just used to set up IC counters.
It configures an ICT to be a counter, starts it, and stores the information about it in CountData[IOCNUMPORTS]
For a more in depth discussion of the callback/trigger/flags see the IOC module
@param ict The ICT to use for a counter
@param trigger the edge trigger to count on
@param load The initial value to load as well as the reload value if auto-reload is on
@param callback the zero crossing callback to use. Set this to NULL if no callback is desired.
@param privatedata void pointer to data passed to the callback
@param flags counter flags
@return SUCCESS or FAILURE
@see IOC_IC
*/
int __attribute__((far))Counter_Ipac(int ict,u8 trigger, u16 load,count_callback callback,void *privatedata, u8 flags);


/*!Change the output % of a filtered pwm DtoA
@param dac the dac to use 0 or 1
@param output the output percentage 0-100
@return SUCCESS
*/
int __attribute__((far))DtoA_pwm(int dac, int output);

/*!Change the output % of a general IPAC (could be either)
@param dac the dac to use, in IPAC 0-1(filtered)2-5(DtoA)
@param output the output percentage 0-100
@return SUCCESS
*/
int __attribute__((far))DtoA_Set(int dac, int percent);

/*!rc
 * "run commands"
 * entry into the end users application
 */
int __attribute__((far)) rc(void);

/*@}*/
#endif

