#ifndef REALTIME_H
#define REALTIME_H

#include <gnu_defs.h>
#include <arch/regdefs.h>

/*! @defgroup realtime Realtime Scheduler

    This module contains functions for implementing a real time scheduler
    The scheduler is intialized by IpacInit and runs on 1.024 ms intervals.
    This scheduler runs with interrupts enabled and therefore can run longish
     tasks without risk of slowing down the rest of the system. It's only danger is
    overflowing the stack, but this can be avoided with some common sense.
    i.e. don't create a self perpetuating task with a period longer than its reschedule time.
    Also, when refreshing the screen remember that VT100 commands and ser_printf are not recursive,
    so they can only run inside one thread at a time.
    @see ser_printf
 */

/*@{*/

/*! type RTIME is used for all time references. It's units are in us*/
typedef unsigned long RTIME;

extern RTIME sys_clock;

/*! type taskdata is passed to each real time task, it is commonly used to pass a pointer value*/
typedef long taskdata;

/*! all real time tasks are of type ptask, no return value with a taskdata variable passed to them
IMPORTANT:
A task passed to the scheduler must have __attribute__((far)) or it will fly off into space when the task is run!
*/
typedef void __attribute__((far))(*ptask)(taskdata);

/**
 * big spinlock used to disable all preemption, 
 * currently this just globally turns off interrupts
 * Recursive locks are not considered because it should never be held
 * that long.
 */
static inline void BIG_LOCK(){ 
__asm("psha");
__asm("tfr ccr,a");
__asm("anda #0x10");
__asm("staa 0x1101");
__asm("pula");
idisable();
}
static inline void BIG_UNLOCK() {if(!(*(u8 *)0x1101))ienable();}

/*!a reat lime task node, this is the type of structure returned by RTaddtask and can be passed directly to RTremtask to delete
a scheduled task before it's run. The total number of available nodes in the system is static and equal to NODESAVAIL.
*/
typedef volatile struct RTaskNode{
volatile struct RTaskNode *previous;
volatile struct RTaskNode *next;
volatile ptask task;/*!task to execute*/
volatile taskdata arg;/*!a single integer arguement*/
volatile RTIME time;/*!time of execution*/
}RTaskNode;


/*!The task lisk head type.
*/
typedef struct RTaskList{

RTaskNode *head;
RTaskNode *tail;
int size;
}RTaskList;



/*!
adds a task to the scheduler

RTaddtask adds a task to the scheduler. The task is a function of type ptask and will be passed a taskdata arguement when it is called.
The taskdata arguement is commonly used to cast a pointer, this will cause the GNU compiler to issue a warning but will work fine.
The final arguement, time, is the absolute time the task is scheduled to run in micro-seconds(us), this will therefore need to be the sum of the current system time
and the offset when the task is supposed to run. The current system time is stored in the global variable systime which will need to be
externally referenced (extern RTIME systime). This variable holds the current system time in us and is updated every 1.024 ms.
see the heartbeat function for a simple example of how a task can run and reschedule itself.


@param task the task to run
@param arg the arguement to pass to the task when it is run
@time the abolute time to run the task in us
@return a pointer to the task node assigned for the task. This can be used to unschedule a task.
@see heartbeat
*/
RTaskNode *__attribute__((far))RTaddtask(ptask task,taskdata arg, RTIME time);

/*! remove a task from the task list
remember that a task is reassigned a node every time it's scheduled, so if a task is rescheduling itself and you want to be able to
stop it you'll need to keep track of the pointers coming back from the RTaddtask call..
@param taskptr the node to pull, this should be the value returned from RTaddtask
*/
void __attribute__((far))RTremove_task(RTaskNode *taskptr);

/*!
Disable Real Time interrupts
*/
void __attribute__((far))RTdisable(void);

/*!
Enable Real Time interrupts
*/
void __attribute__((far))RTenable(void);

/*!
Disable Real Time interrupts and save their current state
*/
void __attribute__((far))RTsaveflags(void);

/*!
Restore the interrupt state saved by RTsaveflags
@see RTsaveflags
*/
void __attribute__((far))RTrestoreflags(void);

/*@}*/
#endif
