/* SER.H
 * Header file for serial drivers for EMAC uP-HC12*/
/*********************************************************************/
/*							SERIAL    								 */
/*********************************************************************/

/*! @defgroup serial Serial Port Control

    This module contains functions from controlling the serial port. This includes string transmission, getch and getline.
    a ser printf is also implemented, but kept at a higher level in the Ipac module. All of these functions depend on Serial_Initialize first
    being called to intialize the serial ports and their data structures. Serial_Initialize is called by IpacInit to set up the serial ports to a
    default of 8BITS 9600 baud.
 */

/*@{*/

#ifndef _SER_H_
#define _SER_H_

#include <GNU_defs.h>

#ifndef SERIALCODE
#define SERIALCODE
#endif

/** Define COM Protocols 
 * legacy, this may not be needed
 */	
#define COM0PROTOCOL	RS232	
#define COM1PROTOCOL	RS232


#define ECHO 1 //echo flag used for ser_getline
#define NO_ECHO 0

#define _USE_INTERRUPTS_

/* Mode flags */	
#define EIGHT_BIT	0x00/* com0 */		
#define NINE_BIT	0x10
#define EVEN_PARITY	0x02
#define ODD_PARITY	0x03
#define NO_PARITY	0

typedef void __attribute__((far))(*SCICallback)(int,void *);


/*! Initialize a serial port.

    This function initiailizes and configures a serial port
    It must be called at least once before the a serial port is used
    After that it can be called at any time to change the serial ports configuration.
    IpacInit calls this once for each port and initializes them to 8BIT 9600 baud.
    @param channel the serial port to address, PORT232 or PORT422
    @param mode flags for the serial port to control the parity and number of bits, see "Mode flags" in the defines
    @param baud rate the serial ports baud rate.
    @return SUCCESS
    @see IpacInit
*/
int __attribute__((far))Serial_Initialize(int channel, char mode, long baud);

/*! transmit one byte over a serial port.

    This function transmits a single byte out of a configured serial port.
    The serial port used must be pre configured bye Serial_Initialize.

    @param channel the serial port to address, PORT232 or PORT422
    @param sendta the char to send
    @return 0
    @see Serial_Initialize
*/
char __attribute__((far)) Ser_Send_Byte(char channel, char sendta);/* does not block */

/*! transmit a string over a serial port.

    This function uses Ser_Send_Byte to transmit a string out of a serial port.
    This happens in the foreground and blocks until the string is sent.

    @param channel the serial port to address, PORT232 or PORT422
    @param *sendta the string to send
    @return 0
    @see Ser_Send_Byte
*/
char __attribute__((far)) Ser_Send_String(char channel, char *sendta);

/*! Pull data from the a serial fifo.

    SCIrx pulls up to count bytes from it's fifo and stores them in the array pointed to
    by buffer. It returns the number of characters successfully pulled from the fifo.
    The serial port must first be Initialized before it can be read from.

    @param port the serial port to address, PORT232 or PORT422
    @param count the number of characters to try to pull.
    @param *buffer pointer to an array of data to store the received characters in
    @return the number of characters stored in the buffer
    @see Serial_Initialize
*/
ssize_t __attribute__((far))SCIrx(int port,int count, u8 *buffer);


/*! Block until a character is received from a serial port.

    Blocks until a character is received from an initialized serial port.

    @param port the serial port to address, PORT232 or PORT422
    @return the character received
    @see Serial_Initialize
*/
char __attribute__((far))ser_getch(int port);

/*! Block until a line is recieved from a serial port

    Blocks until a line(terminated by a linefeed) is received.
    The linefeed is then overwritten with a zero to make it a string and
    placed in the array pointed to by buffer.

    @param port the serial port to address, PORT232 or PORT422
    @param buffer pointer to an array to store the line in.
    @param max the maximum number of characters that the line can contain.
    @param echo ECHO or NO_ECHO setting echo causes characters received to "echo" to the serial port.
    @return the number of characters received
*/
int __attribute__((far))ser_getline(int port,char *buffer, int max,int echo);

/*! Enable the Transmitter (Hardware handshaking and 422)

    Sets RTS active and inserts a short delay to allow transmitter hardware to turn on if it exists.

    @param port the serial port to address, PORT232 or PORT422
    @return SUCCESS
*/
int __attribute__((far))SCItx_En(int port);

/*! Disable the Transmitter (Hardware handshaking and 422)

    Waits for any pending serial transmissions to finish and sets RTS inactive

    @param port the serial port to address, PORT232 or PORT422
    @return SUCCESS
*/
int __attribute__((far))SCItx_Dis(int port);

int __attribute__((far)) Serial_Init(char channel, char mode, long baud, 
	char snd_en, char rcv_en, char snd_int, char rcv_int);
ssize_t __attribute__((far)) SCItx(int port,int count, char *buffer);
void __attribute__((far))SCIdisable(int port);
void __attribute__((far))SCIenable(int port);
int __attribute__((far)) SCI_IdleTick(int port);

/*@}*/



#endif


