#ifndef SPICODE
#define SPICODE

#include "Fifo.h"

/*! @defgroup SPI SPI Control Module

    This module contains functions for controlling the IPACs internal SPI in a blocking or interrupt based system.
    Currently only the blocking mode is used because the SPI is fast enough that it only blocks for a couple micro-seconds.
    And background tasks are still running during this time.
 */

/*@{*/

/*!Motorola CPHA clock phase flag*/
#define CPHA	0x04	//trailing edge sample
/*!Motorola CPOL clock polarity flag*/
#define CPOL	0x08	//high idle

/*!SPI Init bitrates*/

#define MHZ4	 0x20  //divide by 6
#define MHZ2  0x21  //divide by 12
#define MHZ1  0x22  //divide by 24

/*!Callback used for spi in interrupt mode
@param data byte read from the SPI
@param privatedata generic user pointer
*/
typedef void __attribute__((far))(*spi_callback)(u8 data,void *privatedata);


/*!Configure the SPI
This function is called to set the SPI's bitrate and mode
@param bitrate MHZ4,MHZ2 or MHZ1 sets the bitrate
@param flags bitmask of CPHA, CPOL or nothing. See the motorala SPI pdf for descriptions of the various modes
@return SUCCESS
*/
int __attribute__((far))SPI0_setup(int bitrate, int flags);

/*!Install a callback and transmit fifo in the interrupt handler
Once this function has been called the interrupt hander will call callback with every data byte received.
Data will be transmitted through fifo as soon as the SPI becomes ready.
@param callback the receive data callback
@param fifo the fifo to transmit data from
@param privatedata generic user pointer
@return SUCCESS
*/
int __attribute__((far))SPI0_IRQ_Control(spi_callback callback, KFifo *fifo,void *privatedata );

/*!Trancieve of buffer of size size
Calling this function will transmit the contents of the buffer out of the SPI in the foreground.
Data is received simultanously overwrites the contents of buffer.
@param buffer the array to transmit from and store data in
@param size the number of characters to transmit from the buffer
@return the number of characters transmitted
*/
int __attribute__((far))SP0_Tranceive_Blocking(u8 *buffer, int size);

/*@}*/

#endif

