/*VT100 control codes
*/

/*! @defgroup IPACTEST An example program for the IPAC

    The IPAC module provides an example user program for the IPAC software package.
    This is a menu based VT100 program that runs over the 232 serial port.
 */


/*screen refresh rates for the various menu programs
These values are used to by real-time tasks that reschedule themselves to determine the offset from systime at which they will
run next.
*/
#define HEARTPERIOD     1000000
#define DIGREFRESH       200000
#define ATODREFRESH    100000
#define COUNTREFRESH  200000
#define TIMEREFRESH      1000000

/*! display information structure
This type of structure is used throughout the program to tell realtime tasks where to display their data.
Typically it's used by creating a local variable of type displaydata, intializing the structure and passing via taskdata to
a realtime task. The task must be killed before the function because after that the data is no longer valid.
*/

typedef struct displaydata{
int port;
int line;
int column;
RTaskNode *task;
} displaydata;


/*! zero crossing data
This type of structure is used by the counter section of the program to store information about when
zerocrossings are happening. this information is then used alongside the systime variable to compute the counter
frequency.
*/
typedef struct zero_crossings
{
int zerocrossings;
RTIME lastzero;
int frequency;
}zerocross;

/*!display real time task
This task displays the time read from the RTC
@param privatedata cast to a pointer to pass a displaydata to it.
*/
void __attribute__((far))displaytime(taskdata privatedata);

/*!heartbeat task
This task flashes the IPACs status LED
@param status starting status ON or OFF
*/
void __attribute__((far))heartbeat(taskdata status);

/*!The top level test program
This is called by main to start the IPAC test
It calles some internal data initialiization functions, displayed the menu and sits in a getch/switch loop.
This function also calls KeyCall to set up a keypad callback and sets up a task to display the RTC on second intervals
@parm port  the serial port to run the test over                                                                                       `
@return this function returns in the case of a port switch, the return value is the new port number to run it on.
*/
int __attribute__((far))IpacTestProg(int port);

/*!The Digital port menu program
This program displays the current digital port readings and configurations
It also provides a command line for writing/configuring the ports
Data is displayed using the Display_Dig_Task real time task and
commands are received in the foreground by ser_getline
@param serial the serial port to run the test over
@param line the starting line number where the test wll be displayed
@param column the starting column number where the test will be displayed
@return 0
*/
int __attribute__((far))Disp_Digital_Input_Readings(int serial, int line, int column);

/*!realtime task for diplaying digital port information
This task is created by Disp_Digital_Input_Readings to display digital data periodically.
It reschedules itself to make itself periodic and is removed from the task queue when  Disp_Digital_Input_Readings exits.
@param privatedata in this case privatedata is used to pass a displaystruct to the task. This is used by the task to set the cursor position
*/
void __attribute__((far))Display_Dig_Task(taskdata privatedata);

/*!Digital command interpreter
This function uses strtoul to interepret a line of data received from the serial port.
This line is then used to call routines in the relay module
@return -1 if q is received -2 for invalid command 0 on success
*/
int __attribute__((far))Digitalcommand(char *line);

/*!The AtoD port menu program
This program displays the Analog to Digital port readings.
Data is displayed using the AtoD callback function AtoDreadings in conjuction with the real time scheduler to start readings.
@param serial the serial port to run the test over
@param line the starting line number where the test wll be displayed
@param column the starting column number where the test will be displayed
@return 0
*/
int __attribute__((far))Disp_Analog_to_Digital_Readings(int serial,int line, int column);


/*!AtoD callback
This callback is registered by Disp_Analog_to_Digital_Readings to display AtoD periodically.
It schedules AtoD conversion cycles and by doing so makes itself periodic.
@param module the AtoD module that's conversion is complete
@param *data_array a pointer to the array of data (u16 AtoD[8)
@param *status_array a pointer to the array of flags(u8 AtoDstatus[8)
@param *pdata a void pointer that in this case is used to pass a displaystruct to the task. This is used by the task to set the cursor position
@see Install_AtoDcallback
*/
void __attribute__((far))AtoDreadings(int module,u16 *data,volatile u8 *status,void *pdata);


/*!The PWM port menu program
This program displays the current PWM settings
It also provides a command line for changing the settings.
This function doesn't use the scheduler, it just sits in a while loop getting data with ser_getline
@param serial the serial port to run the test over
@param line the starting line number where the test wll be displayed
@param column the starting column number where the test will be displayed
@return 0
*/
int __attribute__((far))PWM_Interface(int port,int line,int column);

/*!PWMmmand interpreter
This function uses strtoul to interepret a line of data received from the serial port.
This line is then used to call the IPAC PWM control function.
@return -1 if q is received -2 for invalid command 0 on success
*/
int __attribute__((far))PWMcommand(char *line);

/*!The Counter menu program
This program information about the counters such as, trigger, count, reload, flags, number of zeros, and the frequency.
It also provides a command line for changing the settings.Data is displayed using the Display_Count_Task real time task and
commands are received in the foreground by ser_getline
@param serial the serial port to run the test over
@param line the starting line number where the test wll be displayed
@param column the starting column number where the test will be displayed
@return 0
*/
int __attribute__((far))Counter_Interface(int port,int line,int column);

/*!realtime task for diplaying counter
This task is created by Counter_Interface to display counter data periodically.
It reschedules itself to make itself periodic and is removed from the task queue when Counter_Interface exits.
@param privatedata in this case privatedata is used to pass a displaystruct to the task. This is used by the task to set the cursor position
*/
void __attribute__((far))Display_Count_Task(taskdata privatedata);

/*!Counter command interpreter
This function uses strtoul to interepret a line of data received from the serial port.
This line is then used to call the IPAC counter control function.
@return -1 if q is received -2 for invalid command 0 on success
*/
int __attribute__((far))Countcommand(char *line) ;

/*!Zero Count callback
This callback is registered by Counter_Interface to update zero crossing information.
It records current time updates the zero counter and approximates the frequency the edges are occuring on.
@param channel the IC channel that caused the callback
@param *privatedata a void pointer that in this case is used to point to a zerocross struct
*/
void __attribute__((far))ZeroCount(int channel, void *privatedata);

/*!The DtoA menu program
This program displays the current filtered PWM DtoA settings
It also provides a command line for changing the settings
@param serial the serial port to run the test over
@param line the starting line number where the test wll be displayed
@param column the starting column number where the test will be displayed
@return 0
*/
int __attribute__((far))DtoA_Interface(int port,int line,int column);

/*!DtoA command interpreter
This function uses strtoul to interepret a line of data received from the serial port.
This line is then used to call the IPAC filtered PWM command.
@return -1 if q is received -2 for invalid command 0 on success
*/
int __attribute__((far))DtoAcommand(char *line);

/*!Keypad Callback
This task is registered by IpacTestProg to display the keys pressed on the keypad.
@param privatedata in this case privatedata is used to pass a displaystruct to the task. This is used by the task to set the cursor position
*/
void __attribute__((far))KeyDisplay(void *privatedata);

/*!The RTC menu program
Set the time of the RTC
@param port the serial port to run the test over
@param line the starting line number where the test wll be displayed
@param column the starting column number where the test will be displayed
@return 0
*/
int __attribute__((far))RTC_Interface(int port,int line,int column);

/*!The LCD menu program
Control various aspects of the LCD
@param port the serial port to run the test over
@param line the starting line number where the test wll be displayed
@param column the starting column number where the test will be displayed
@return 0
*/
int __attribute__((far))LCD_control(int port,int line,int column);


void __attribute__((far))Display_ExtAn_Task(taskdata privatedata);

/*@}*/

